
/**
 * @vue/test-utils v2.0.0-alpha.1
 * (c) 2020 Lachlan Miller
 * Released under the MIT License
 */

import { nextTick, getCurrentInstance, reactive, defineComponent, h, createApp } from 'vue';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

var ErrorWrapper = /** @class */ (function () {
    function ErrorWrapper(_a) {
        var selector = _a.selector;
        this.selector = selector;
    }
    ErrorWrapper.prototype.wrapperError = function (method) {
        return Error("Cannot call " + method + " on an empty wrapper.");
    };
    ErrorWrapper.prototype.attributes = function () {
        throw this.wrapperError('attributes');
    };
    ErrorWrapper.prototype.classes = function () {
        throw this.wrapperError('classes');
    };
    ErrorWrapper.prototype.exists = function () {
        return false;
    };
    ErrorWrapper.prototype.find = function () {
        throw this.wrapperError('find');
    };
    ErrorWrapper.prototype.findAll = function () {
        throw this.wrapperError('findAll');
    };
    ErrorWrapper.prototype.setChecked = function () {
        throw this.wrapperError('setChecked');
    };
    ErrorWrapper.prototype.setValue = function () {
        throw this.wrapperError('setValue');
    };
    ErrorWrapper.prototype.text = function () {
        throw this.wrapperError('text');
    };
    ErrorWrapper.prototype.trigger = function () {
        throw this.wrapperError('trigger');
    };
    return ErrorWrapper;
}());

var DOMWrapper = /** @class */ (function () {
    function DOMWrapper(element) {
        this.element = element;
    }
    DOMWrapper.prototype.classes = function (className) {
        var classes = this.element.classList;
        if (className)
            return classes.contains(className);
        return Array.from(classes);
    };
    DOMWrapper.prototype.attributes = function (key) {
        var attributes = this.element.attributes;
        var attributeMap = {};
        for (var i = 0; i < attributes.length; i++) {
            var att = attributes.item(i);
            attributeMap[att.localName] = att.value;
        }
        return key ? attributeMap[key] : attributeMap;
    };
    DOMWrapper.prototype.exists = function () {
        return true;
    };
    DOMWrapper.prototype.text = function () {
        var _a;
        return (_a = this.element.textContent) === null || _a === void 0 ? void 0 : _a.trim();
    };
    DOMWrapper.prototype.html = function () {
        return this.element.outerHTML;
    };
    DOMWrapper.prototype.find = function (selector) {
        var result = this.element.querySelector(selector);
        if (result) {
            return new DOMWrapper(result);
        }
        return new ErrorWrapper({ selector: selector });
    };
    DOMWrapper.prototype.findAll = function (selector) {
        return Array.from(this.element.querySelectorAll(selector)).map(function (x) { return new DOMWrapper(x); });
    };
    DOMWrapper.prototype.setChecked = function (checked) {
        if (checked === void 0) { checked = true; }
        return __awaiter(this, void 0, void 0, function () {
            var element, type;
            return __generator(this, function (_a) {
                element = this.element;
                type = this.attributes().type;
                if (type === 'radio' && !checked) {
                    throw Error("wrapper.setChecked() cannot be called with parameter false on a '<input type=\"radio\" /> element.");
                }
                // we do not want to trigger an event if the user
                // attempting set the same value twice
                // this is beacuse in a browser setting checked = true when it is
                // already true is a no-op; no change event is triggered
                if (checked === element.checked) {
                    return [2 /*return*/];
                }
                element.checked = checked;
                return [2 /*return*/, this.trigger('change')];
            });
        });
    };
    DOMWrapper.prototype.setValue = function (value) {
        var element = this.element;
        var tagName = element.tagName;
        var type = this.attributes().type;
        if (tagName === 'OPTION') {
            return this.setSelected();
        }
        else if (tagName === 'INPUT' && type === 'checkbox') {
            return this.setChecked(value);
        }
        else if (tagName === 'INPUT' && type === 'radio') {
            return this.setChecked(value);
        }
        else if (tagName === 'INPUT' ||
            tagName === 'TEXTAREA' ||
            tagName === 'SELECT') {
            element.value = value;
            if (tagName === 'SELECT') {
                return this.trigger('change');
            }
            this.trigger('input');
            // trigger `change` for `v-model.lazy`
            return this.trigger('change');
        }
        else {
            throw Error("wrapper.setValue() cannot be called on " + tagName);
        }
    };
    DOMWrapper.prototype.setSelected = function () {
        var element = this.element;
        if (element.selected) {
            return;
        }
        element.selected = true;
        var parentElement = element.parentElement;
        if (parentElement.tagName === 'OPTGROUP') {
            parentElement = parentElement.parentElement;
        }
        return new DOMWrapper(parentElement).trigger('change');
    };
    DOMWrapper.prototype.trigger = function (eventString) {
        return __awaiter(this, void 0, void 0, function () {
            var evt;
            return __generator(this, function (_a) {
                evt = document.createEvent('Event');
                evt.initEvent(eventString);
                if (this.element) {
                    this.element.dispatchEvent(evt);
                }
                return [2 /*return*/, nextTick];
            });
        });
    };
    return DOMWrapper;
}());

var MOUNT_ELEMENT_ID = 'app';

var VueWrapper = /** @class */ (function () {
    function VueWrapper(vm, events, setProps) {
        this.__emitted = {};
        this.__vm = vm;
        this.__setProps = setProps;
        this.componentVM = this.vm.$refs['VTU_COMPONENT'];
        this.__emitted = events;
    }
    Object.defineProperty(VueWrapper.prototype, "appRootNode", {
        get: function () {
            return document.getElementById(MOUNT_ELEMENT_ID);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VueWrapper.prototype, "hasMultipleRoots", {
        get: function () {
            // if the subtree is an array of children, we have multiple root nodes
            return this.componentVM.$.subTree.shapeFlag === 16 /* ARRAY_CHILDREN */;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VueWrapper.prototype, "parentElement", {
        get: function () {
            return this.componentVM.$el.parentElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VueWrapper.prototype, "element", {
        get: function () {
            // if the component has multiple root elements, we use the parent's element
            return this.hasMultipleRoots ? this.parentElement : this.componentVM.$el;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VueWrapper.prototype, "vm", {
        get: function () {
            return this.__vm;
        },
        enumerable: true,
        configurable: true
    });
    VueWrapper.prototype.classes = function (className) {
        return new DOMWrapper(this.element).classes(className);
    };
    VueWrapper.prototype.attributes = function (key) {
        return new DOMWrapper(this.element).attributes(key);
    };
    VueWrapper.prototype.exists = function () {
        return true;
    };
    VueWrapper.prototype.emitted = function () {
        return this.__emitted;
    };
    VueWrapper.prototype.html = function () {
        return this.appRootNode.innerHTML;
    };
    VueWrapper.prototype.text = function () {
        var _a;
        return (_a = this.element.textContent) === null || _a === void 0 ? void 0 : _a.trim();
    };
    VueWrapper.prototype.find = function (selector) {
        // force using the parentElement to allow finding the root element
        var result = this.parentElement.querySelector(selector);
        if (result) {
            return new DOMWrapper(result);
        }
        return new ErrorWrapper({ selector: selector });
    };
    VueWrapper.prototype.findAll = function (selector) {
        var results = this.appRootNode.querySelectorAll(selector);
        return Array.from(results).map(function (x) { return new DOMWrapper(x); });
    };
    VueWrapper.prototype.setProps = function (props) {
        this.__setProps(props);
        return nextTick();
    };
    VueWrapper.prototype.trigger = function (eventString) {
        var rootElementWrapper = new DOMWrapper(this.element);
        return rootElementWrapper.trigger(eventString);
    };
    return VueWrapper;
}());
function createWrapper(vm, events, setProps) {
    return new VueWrapper(vm, events, setProps);
}

var createEmitMixin = function () {
    var events = {};
    var emitMixin = {
        beforeCreate: function () {
            var originalEmit = getCurrentInstance().emit;
            getCurrentInstance().emit = function (event) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                events[event]
                    ? (events[event] = __spreadArrays(events[event], [__spreadArrays(args)]))
                    : (events[event] = [__spreadArrays(args)]);
                return originalEmit.call.apply(originalEmit, __spreadArrays([getCurrentInstance(), event], args));
            };
        }
    };
    return {
        events: events,
        emitMixin: emitMixin
    };
};

var createDataMixin = function (data) {
    return {
        created: function () {
            var _a;
            for (var _i = 0, _b = Object.entries(data); _i < _b.length; _i++) {
                var _c = _b[_i], k = _c[0], v = _c[1];
                getCurrentInstance().data = __assign(__assign({}, getCurrentInstance().data), (_a = {}, _a[k] = v, _a));
            }
        }
    };
};

function mount(originalComponent, options) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y;
    var component = __assign({}, originalComponent);
    // Reset the document.body
    document.getElementsByTagName('html')[0].innerHTML = '';
    var el = document.createElement('div');
    el.id = MOUNT_ELEMENT_ID;
    document.body.appendChild(el);
    // handle any slots passed via mounting options
    var slots = ((_a = options) === null || _a === void 0 ? void 0 : _a.slots) &&
        Object.entries(options.slots).reduce(function (acc, _a) {
            var name = _a[0], slot = _a[1];
            // case of an SFC getting passed
            if (typeof slot === 'object' && 'render' in slot) {
                acc[name] = slot.render;
                return acc;
            }
            acc[name] = function () { return slot; };
            return acc;
        }, {});
    // override component data with mounting options data
    if ((_b = options) === null || _b === void 0 ? void 0 : _b.data) {
        var dataMixin = createDataMixin(options.data());
        component.mixins = __spreadArrays((component.mixins || []), [dataMixin]);
    }
    // we define props as reactive so that way when we update them with `setProps`
    // Vue's reactivity system will cause a rerender.
    var props = reactive(__assign(__assign({}, (_c = options) === null || _c === void 0 ? void 0 : _c.props), { ref: 'VTU_COMPONENT' }));
    // create the wrapper component
    var Parent = defineComponent({
        render: function () {
            return h(component, props, slots);
        }
    });
    var setProps = function (newProps) {
        for (var _i = 0, _a = Object.entries(newProps); _i < _a.length; _i++) {
            var _b = _a[_i], k = _b[0], v = _b[1];
            props[k] = v;
        }
        return app.$nextTick();
    };
    // create the vm
    var vm = createApp(Parent);
    // global mocks mixin
    if ((_e = (_d = options) === null || _d === void 0 ? void 0 : _d.global) === null || _e === void 0 ? void 0 : _e.mocks) {
        var mixin = {
            beforeCreate: function () {
                var _a;
                for (var _i = 0, _b = Object.entries((_a = options.global) === null || _a === void 0 ? void 0 : _a.mocks); _i < _b.length; _i++) {
                    var _c = _b[_i], k = _c[0], v = _c[1];
                    this[k] = v;
                }
            }
        };
        vm.mixin(mixin);
    }
    // use and plugins from mounting options
    if ((_g = (_f = options) === null || _f === void 0 ? void 0 : _f.global) === null || _g === void 0 ? void 0 : _g.plugins) {
        for (var _i = 0, _z = (_j = (_h = options) === null || _h === void 0 ? void 0 : _h.global) === null || _j === void 0 ? void 0 : _j.plugins; _i < _z.length; _i++) {
            var use = _z[_i];
            vm.use(use);
        }
    }
    // use any mixins from mounting options
    if ((_l = (_k = options) === null || _k === void 0 ? void 0 : _k.global) === null || _l === void 0 ? void 0 : _l.mixins) {
        for (var _0 = 0, _1 = (_o = (_m = options) === null || _m === void 0 ? void 0 : _m.global) === null || _o === void 0 ? void 0 : _o.mixins; _0 < _1.length; _0++) {
            var mixin = _1[_0];
            vm.mixin(mixin);
        }
    }
    if ((_q = (_p = options) === null || _p === void 0 ? void 0 : _p.global) === null || _q === void 0 ? void 0 : _q.components) {
        for (var _2 = 0, _3 = Object.keys((_s = (_r = options) === null || _r === void 0 ? void 0 : _r.global) === null || _s === void 0 ? void 0 : _s.components); _2 < _3.length; _2++) {
            var key = _3[_2];
            vm.component(key, options.global.components[key]);
        }
    }
    if ((_u = (_t = options) === null || _t === void 0 ? void 0 : _t.global) === null || _u === void 0 ? void 0 : _u.directives) {
        for (var _4 = 0, _5 = Object.keys((_w = (_v = options) === null || _v === void 0 ? void 0 : _v.global) === null || _w === void 0 ? void 0 : _w.directives); _4 < _5.length; _4++) {
            var key = _5[_4];
            vm.directive(key, options.global.directives[key]);
        }
    }
    // provide any values passed via provides mounting option
    if ((_y = (_x = options) === null || _x === void 0 ? void 0 : _x.global) === null || _y === void 0 ? void 0 : _y.provide) {
        for (var _6 = 0, _7 = Reflect.ownKeys(options.global.provide); _6 < _7.length; _6++) {
            var key = _7[_6];
            // @ts-ignore: https://github.com/microsoft/TypeScript/issues/1863
            vm.provide(key, options.global.provide[key]);
        }
    }
    // add tracking for emitted events
    var _8 = createEmitMixin(), emitMixin = _8.emitMixin, events = _8.events;
    vm.mixin(emitMixin);
    // mount the app!
    var app = vm.mount(el);
    return createWrapper(app, events, setProps);
}

// TODO: Borrow typings from vue-router-next
var RouterLinkStub = defineComponent({
    name: 'RouterLinkStub',
    props: {
        to: {
            type: [String, Object],
            required: true
        }
    },
    render: function () {
        return h('a', undefined, this.$slots.default());
    }
});

export { RouterLinkStub, mount };
